<?php

use Cashbene\Core\Dto\Payment\PayByPayu as PaymentDTO;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Cashbene\Core\Utils\CredentialsSession;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Cashbene\GatewayPrestashop\Rest\CashbeneAbstractRestController;
use \Cashbene\GatewayPrestashop\Service\CheckoutService;

class CashbenePaymentGatewayPaymentModuleFrontController extends CashbeneAbstractRestController
{
    const POST_ACTIONS = ['storeCardToken', 'pay'];
    const GET_ACTIONS = ['fetchMethods'];
    const DELETE_ACTIONS = ['deleteStoredCardToken'];

    protected function processGetRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'fetchMethods':
                return $this->fetchPaymentMethods();
        }
    }

    /**
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see PaymentContext::fetchPaymentMethods()
     */
    private function fetchPaymentMethods() {
        try {
            $data = self::$cashbeneGateway->paymentContext()->fetchPaymentMethods(CredentialsSession::getCredentials(self::$cashbeneGateway));
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    protected function processPostRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'storeCardToken':
                return $this->storeCardToken($request);
            case 'pay':
                return $this->payByPayu($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see PaymentContext::storeCardToken()
     */
    private function storeCardToken(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);
            /** @var PaymentDTO $token */
            $token = self::$serializer->denormalize(
                $params, PaymentDTO::class, 'array'
            );

            $data = self::$cashbeneGateway->paymentContext()->storeCardToken(CredentialsSession::getCredentials(self::$cashbeneGateway), $token);
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see PaymentContext::payByPayu()
     */
    private function payByPayu(Request $request) {
        try {
            $params = json_decode($request->getContent(), true);
            $credentials = CredentialsSession::getCredentials(self::$cashbeneGateway);
            $payment = self::$serializer->denormalize(
                $params, PaymentDTO::class, 'array'
            );

            $data = self::$cashbeneGateway
                ->paymentContext()
                ->payByPayu($credentials, $payment, $request->get('checkoutId'));

            if ($data->status === 'SUCCESSFUL') {
                CheckoutService::destroyCart();
            }
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($data, 201);
    }

    protected function processPutRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }

    protected function processDeleteRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'deleteStoredCardToken':
                return $this->deleteStoredCardToken($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see PaymentContext::deleteCardToken()
     */
    public function deleteStoredCardToken(Request $request)
    {
        try {
            self::$cashbeneGateway->paymentContext()->deleteCardToken(CredentialsSession::getCredentials(self::$cashbeneGateway), $request->get('token'));
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success(null, 204);
    }

}
