<?php

use Cashbene\Core\Dto\Regulation\MerchantRegulation;
use Cashbene\Core\Exception\HttpExceptionInterface;
use Cashbene\Core\Exception\ResourceNotFoundErrorException;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Cashbene\GatewayPrestashop\Rest\CashbeneAbstractRestController;
use Cashbene\GatewayPrestashop\Service\RegulationService;

class CashbenePaymentGatewayRegulationModuleFrontController extends CashbeneAbstractRestController
{
    const GET_ACTIONS = ['getCashbeneTerm', 'getMerchantTerm'];

    protected function processGetRequest(string $action, Request $request)
    {
        switch ($action) {
            case 'getCashbeneTerm':
                return $this->getCashbeneTerm($request);
            case 'getMerchantTerm':
                return $this->getMerchantTerm($request);
        }
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see RegulationContext::getTerm()
     */
    private function getCashbeneTerm(Request $request) {
        try {
            $result = self::$cashbeneGateway->regulationContext()->getTerm($request->get('type'));
            if ($request->headers->get('Content-Type') != 'application/json') {
                RegulationService::decodePdfBase64ToFile($result->file, $result->termsType);

                /**
                 * die() to skip change header in
                 * @see Symfony\Component\HttpFoundation\Response::sendHeaders();
                 */
                die();
            }
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        return $this->success($result);
    }

    /**
     * @param Request $request
     * @return JsonResponse
     * @throws HttpExceptionInterface
     * @see MerchantRegulation::TYPE_PRIVACY_POLICY_PAGE, MerchantRegulation::TYPE_TERMS_AND_CONDITIONS_PAGE
     */
    private function getMerchantTerm(Request $request) {
        try {
            $result = RegulationService::getMerchantTerm($request->get('type'));
        } catch (HttpExceptionInterface $httpException) {
            return $this->error($httpException);
        }

        if ($result === null) {
            return $this->error(new ResourceNotFoundErrorException(), 404);
        }

        $dto = self::$serializer->denormalize(
            $result,
            MerchantRegulation::class,
            'array'
        );

        return $this->success($dto);
    }

    protected function processPostRequest(string $action, Request $request)
    {
        // TODO: Implement processPostRequest() method.
    }

    protected function processPutRequest(string $action, Request $request)
    {
        // TODO: Implement processPutRequest() method.
    }

    protected function processDeleteRequest(string $action, Request $request)
    {
        // TODO: Implement processDeleteRequest() method.
    }

}
