<?php

namespace Cashbene\GatewayPrestashop\Service;

use Configuration;
use Carrier;
use Group;
use RangeWeight;

/**
 * https://belvg.com/blog/how-to-create-shipping-module-for-prestashop.html
 */
class CarrierService {

    const PREFIX = 'Cashbene_Carrier';
    const CARRIER_NAME = 'InPost - Cashbene payment gateway';

    public function createCarrier()
    {
        // check if carrier exist already
        if ($this->getCarrier()) {
            return;
        }
        $carrier = new Carrier();
        $carrier->name = self::CARRIER_NAME;
        $carrier->active = FALSE;
        $carrier->deleted = 0;
        $carrier->shipping_handling = FALSE;
        $carrier->range_behavior = 0;
        $carrier->delay[Configuration::get('PS_LANG_DEFAULT')] = '2 dni robocze';
        $carrier->shipping_external = FALSE;
        $carrier->need_range = FALSE;
        $carrier->save();

        // need to be called after save
        $carrier->setTaxRulesGroup(1); // 1 is 23%
        $carrier->addZone(1); // 1 is zone for Europe

        $carrier->id_reference = $carrier->id;
        $this->addGroups($carrier);
        $this->addRangeWeight($carrier);

        Configuration::updateValue(self::PREFIX . '_ID', $carrier->id_reference);
    }

    public function getCarrier()
    {
        $carrierId = null;

        // get carrier by ID reference
        $carriers = Carrier::getCarriers(Configuration::get('PS_LANG_DEFAULT'));
        foreach ($carriers as $element) {
            if ($element['id_reference'] == Configuration::get(self::PREFIX . '_ID')) {
                $carrierId = $element['id_carrier'];
            }
        }

        // get carrier By ID
        $carrier = new Carrier($carrierId);
        if ($carrier->id_reference == null || $carrier->deleted == 1) {
            return null;
        }

        return $carrier;
    }

    private function addRangeWeight(Carrier $carrier) {
        $rangeWeight = new RangeWeight();
        $rangeWeight->id_carrier = $carrier->id;
        $rangeWeight->delimiter1 = '0';
        $rangeWeight->delimiter2 = '1000000';
        $rangeWeight->add();
    }

    private function addGroups(Carrier $carrier) {
        $groupIds = [];
        $groups = Group::getGroups(true);
        foreach ($groups as $group) {
            $groupIds[] = $group['id_group'];
        }

        $carrier->setGroups($groupIds);
        $carrier->save();
    }

}
