<?php

namespace Cashbene\GatewayPrestashop\Service;

use Cashbene\Core\Dto\Checkout\CheckoutData;
use Cashbene\Core\Dto\Checkout\CheckoutOrder;
use Cashbene\Core\Dto\ShippingMethodOutput;
use Cashbene\Core\Dto\Shop\Price;
use Cashbene\Core\Dto\Shop\Product;
use Cashbene\Core\Dto\User;
use Cashbene\Core\Utils\CredentialsSession;
use Cashbene\GatewayPrestashop\Rest\CashbeneAbstractRestController;
use Cashbene\GatewayPrestashop\Utils\Lang;
use CashbenePaymentGateway;

class CheckoutService {

	/**
	 * @param Product[] $products
	 * @param string $shippingAddressId
	 * @param string $shippingMethodId
	 * @param string $merchantId
	 * @param string|null $externalOrderId
	 *
	 * @return CheckoutOrder
	 */
	static public function prepareOrder(
		array $products,
		string $shippingAddressId,
		string $shippingMethodId,
		string $merchantId,
		?string $externalOrderId = null
	) {
		$order = new CheckoutOrder();
        $order->shippingAddressId = $shippingAddressId;

        $checkoutData = new CheckoutData();
        $checkoutData->shippingMethodId = $shippingMethodId;

        if ($externalOrderId !== null) {
            $checkoutData->externalOrderId = $externalOrderId;
        }


        $checkoutData->products = self::reCalculateProducts($products);

        $order->checkoutData = (object)[
            $merchantId => $checkoutData
        ];

        return $order;
	}

	/**
	 * @param Product[] $products
	 * @return Product[]
     * // todo add logic if needed, now it was never do anything.
	 */
	private static function reCalculateProducts(array &$products)
	{
//		foreach ($products as $product) {
//		}
		return $products;
	}


    // https://stackoverflow.com/questions/35314410/create-order-in-prestashop-programmatically
    // https://devdocs.prestashop.com/1.7/modules/concepts/controllers/front-controllers/#execution-order-of-the-controllers-functions
    // https://devdocs.prestashop.com/1.7/development/orders-lifecycle/#filling-the-cart-and-creating-the-order
    // http://penguin-arts.com/how-to-create-new-customer-address-programatically-in-prestashop/
    // https://devdocs.prestashop.com/1.7/webservice/resources/orders/#order
    // https://www.prestashop.com/forums/topic/611055-programatically-create-order-and-advanced-stock-management/
    // https://penguin-arts.com/how-to-create-order-in-prestashop-programatically-php/
	public static function createPrestaOrder(
		User $user,
		array $products,
		string $shippingAddressId,
		string $shippingMethodId,
        $singleProductCheckout = false,
        $prices
	)
	{
        $context = \Context::getContext();

        $carrier = self::getCarrier($shippingMethodId);
        $idCarrier = self::getCarrierId($shippingMethodId);

        if (is_null($idCarrier)) {
            throw new \Exception("Shipping method doesn't exists.", 500);
        }

        $cashbeneShippingMethod = $context->controller->get('cashbene.payment.gateway')
            ->getGateway()->shopContext()->getMerchantShippingMethod($shippingMethodId);
        // Carrier end

        // Main order fields start
        $cart = self::createNewCart($products, $user, $singleProductCheckout);
		$order = new \Order();
        $order->reference = \Order::generateReference();
        $order->secure_key = md5(uniqid(mt_rand(0, mt_getrandmax()), true));
        $order->conversion_rate = $context->currency->conversion_rate;
        $order->round_mode = \Configuration::get('PS_PRICE_ROUND_MODE');
        $order->round_type = \Configuration::get('PS_ROUND_TYPE');
        $order->id_cart = $cart->id;
        $order->id_carrier = $idCarrier;
        $order->id_customer = $cart->id_customer;
        $order->id_currency = $cart->id_currency;
        $order->id_lang = (int)$cart->id_lang;
        $order->id_shop = $cart->id_shop;
        $order->id_shop_group = $cart->id_shop_group;
        // Main order fields end

        // Payment start
        $order->payment = "Cashbene Payment";
        $order->module = CashbenePaymentGateway::MODULE_NAME;
        // Payment end

        // Address start
        $prepareAddress = self::prepareAddress($user, $shippingAddressId);
        $address = self::createPrestaAddress($prepareAddress);
        $order->id_address_delivery = $address->id;

        if (isset($prepareAddress['point_id'])) {
            $billingAddress = self::createPrestaAddress(self::getBillingAddress($user));
        } else {
            $billingAddress = $address;
        }
        $order->id_address_invoice = $billingAddress->id;
        // Address end

        $order->product_list = $cart->getProducts();
//        $order->valid = 1;

        //Products price start
        $order->total_products = (float)$cart->getOrderTotal(false, \Cart::ONLY_PRODUCTS, $order->product_list, $idCarrier);
        $order->total_products_wt = (float)$cart->getOrderTotal(true, \Cart::ONLY_PRODUCTS, $order->product_list, $idCarrier);
        $order->total_discounts_tax_excl = (float)abs($cart->getOrderTotal(false, \Cart::ONLY_DISCOUNTS, $order->product_list, $idCarrier));
        $order->total_discounts_tax_incl = (float)abs($cart->getOrderTotal(true, \Cart::ONLY_DISCOUNTS, $order->product_list, $idCarrier));
        $order->total_discounts = $order->total_discounts_tax_incl;
        //Products price end

        $shippingPrice = isset($prices['shippingPrice']) ? $prices['shippingPrice']->amount : $cashbeneShippingMethod->price->amount;

        //Shipping price start
        $order->total_shipping_tax_excl = \Tools::ps_round(
            (float) $shippingPrice / ($carrier->getTaxesRate() / 100 + 1),
            $context->currency->precision
        );
        $order->total_shipping_tax_incl = (float) $shippingPrice;

        if (\Configuration::get('PS_TAX')) {
            $order->total_shipping = $order->total_shipping_tax_incl;
        } else {
            $order->total_shipping = $order->total_shipping_tax_excl;
        }
        //Shipping price end

        // TODO: check wrapping prices
        // Total wrapping price start
        $order->total_wrapping_tax_excl = (float)abs($cart->getOrderTotal(false, \Cart::ONLY_WRAPPING, $order->product_list, $idCarrier));
        $order->total_wrapping_tax_incl = (float)abs($cart->getOrderTotal(true, \Cart::ONLY_WRAPPING, $order->product_list, $idCarrier));
        $order->total_wrapping = $order->total_wrapping_tax_incl;
        // Total wrapping price end

        // Total paid start
        //$order->total_paid_tax_excl = \Tools::ps_round((float)$cart->getOrderTotal(false, \Cart::BOTH, $order->product_list, $idCarrier), $context->currency->precision);
        //$order->total_paid_tax_incl = \Tools::ps_round((float)$cart->getOrderTotal(true, \Cart::BOTH, $order->product_list, $idCarrier),  $context->currency->precision);

        $order->total_paid_tax_excl =
            \Tools::ps_round($order->total_products + $order->total_shipping_tax_excl - $order->total_discounts_tax_excl, $context->currency->precision);
        $order->total_paid_tax_incl =
            \Tools::ps_round($order->total_products_wt + $order->total_shipping_tax_incl - $order->total_discounts_tax_incl, $context->currency->precision);


        if (\Configuration::get('PS_TAX')) {
            $order->total_paid = $order->total_paid_tax_incl;
            $order->total_paid_real = $order->total_paid_tax_incl;
        } else {
            $order->total_paid = $order->total_paid_tax_excl;
            $order->total_paid_real = $order->total_paid_tax_excl;
        }
        // Total paid end

//        $order->invoice_date = $order_date;
//        $order->delivery_date = $order_date;
//        $order->setInvoice(true);

        $order->add();
        $order->setCurrentState(\Configuration::get('PS_CHECKOUT_STATE_WAITING_CASHBENE_PAYMENT'));

        if (isset($prepareAddress['point_id'])) {
            self::setOrderParcelLocker($order->id, $prepareAddress);
        }

        // Insert new Order detail list using cart for the current order
        $order_detail = new \OrderDetail(null, null, $context);
        $order_detail->createList($order, $cart, \Configuration::get('PS_CHECKOUT_STATE_WAITING_CASHBENE_PAYMENT'), $order->product_list);

        // OrderCarrier start
        $order_carrier = new \OrderCarrier();
        $order_carrier->id_order = $order->id;
        $order_carrier->id_carrier = $idCarrier;
        $order_carrier->weight = $order->getTotalWeight();
        $order_carrier->shipping_cost_tax_excl = $order->total_shipping_tax_excl;
        $order_carrier->shipping_cost_tax_incl = $order->total_shipping_tax_incl;
        $order_carrier->add();
        // OrderCarrier end

		return $order;
	}

    /**
     * @param User|null $user
     * @param Product[] $products
     * @param string $shippingAddressId
     * @param string $shippingMethodId
     * @return bool|\Cart
     * @throws \PrestaShopDatabaseException
     * @throws \PrestaShopException
     */
    static function createNewCart(array $products, ?User $user, $singleProductCheckout = false)
    {
        $context = \Context::getContext();

        $cart = $context->cart;
        $cart->id_lang = (int)$context->cookie->id_lang;
        $cart->id_currency = (int)$context->cookie->id_currency;
        $cart->id_shop_group = (int)$context->shop->id_shop_group;
        $cart->id_shop = $context->shop->id;

        if ($singleProductCheckout) {
           $cart->add();
           $context->cookie->id_cart = (int)($cart->id);
            /** @var \Product $product */
            foreach ($products as $product) {
                $cart->updateQty(
                    $product->quantity,
                    $product->productId,
                    $product->externalProductId
                );
            }
        }
        $cart->update();

        if ($user) {
            $customerId = \Customer::customerExists($user->email, true, false);
            if (!$customerId) {
                // Creates new customer
                $customer = new \Customer();
                $customer->email = $user->email;
                $customer->firstname = $user->firstName;
                $customer->lastname =  $user->lastName;
                $customer->passwd = false;

                // Sets quest group for user
                $customer->cleanGroups();
                $customer->id_default_group = \Configuration::get('PS_GUEST_GROUP');
                $customer->addGroups([\Configuration::get('PS_GUEST_GROUP')]);

                $customer->save();
                $customerId = \Customer::customerExists($user->email, true, false);
            }
            $cart->id_customer = $customerId;
        }

        return $cart;
    }

    private static function createPrestaAddress(array $address)
    {
        $context = \Context::getContext();
        $addressObject = new \Address();
        $addressObject->id_customer = $context->customer->id;
        $addressObject->firstname = pSQL($address['first_name']);
        $addressObject->lastname = pSQL($address['last_name']);
        $addressObject->address1 = pSQL($address['address_1']);
        $addressObject->address2 = pSQL($address['address_2']);
        $addressObject->postcode = pSQL($address['postcode']);
        $addressObject->city = pSQL($address['city']);
        $addressObject->id_country = (int)\Country::getByIso($address['country']);
        $addressObject->alias = pSQL($address['point_id'] ?? $address['name']);
        $addressObject->phone = pSQL($address['phone']);

        $state = \State::getIdByName($address['state']);
        if($state) {
            $addressObject->id_state = $state;
        }

        $addressObject->add();

        return $addressObject;
    }

    /**
     * Add parcel locker to order in chosen shipment integration
     *
     * @param $orderId
     * @param $address
     */
    private static function setOrderParcelLocker($orderId, $address)
    {
        $lockerId = $address['point_id'];
        $lockerAddress1 = $address['address_1'];
        $lockerAddress1 .= $address['address_2'] != '' ? ' / '. $address['address_2'] : '';
        $lockerAddress2 =  $address['postcode'].' '.$address['city'];
        $val = $lockerId . ' , ' . $lockerAddress1 . ' , ' . $lockerAddress2;

        InpostIntegrationService::addOrderCashbeneShipping($orderId, $val);
    }

    /**
     * @param User $user
     * @return array|null
     * @throws \Exception
     */
    public static function getBillingAddress(User $user)
    {
        $context = \Context::getContext();
        $gateway = $context->controller->get('cashbene.payment.gateway')
            ->getGateway();
        $addresses = $gateway->userContext()->getShipmentAddresses(
            CredentialsSession::getCredentials($gateway),
            $user->uuid,
            ShippingMethodOutput::TYPE_COURIER
        );

        foreach ($addresses as $address) {
            if ($address->defaultShipment === true) {
                return self::prepareAddressArray($user, $address);
            }
        }

		// If no default address, return first address
        foreach ($addresses as $address) {
            if ($address->type === 'COURIER') {
                return self::prepareAddressArray($user, $address);
            }
        }

        return null;
    }

    /**
     * @param User $user
     * @param $shipmentAddressId
     * @return array
     * @throws \Exception
     */
	private static function prepareAddress(User $user, $shipmentAddressId, $additionalFields = false)
	{
		$gateway = CashbeneAbstractRestController::$cashbeneGateway;
		$address = $gateway->userContext()->getShipmentAddress(
			CredentialsSession::getCredentials($gateway),
			$user->uuid,
			$shipmentAddressId
		);

        if ($address->type === ShippingMethodOutput::TYPE_POINT) {
            $additionalFields = true;
        }

		return self::prepareAddressArray($user, $address, $additionalFields);
	}

    /**
     * @param User $user
     * @param ShippingMethodOutput $address
     * @return array
     */
    private static function prepareAddressArray(User $user, $address, $additionalFields = false)
    {
        $data = [
            'first_name' => $user->firstName,
            'last_name' => $user->lastName,
            'email' => $user->email,
            'phone' => $user->mobile ? $user->mobile->prefix.$user->mobile->number : null,
            'address_1' => $address->address->streetName.' '.$address->address->streetNumber,
            'address_2' => $address->address->flatNumber,
            'city' => $address->address->city,
            'state' => $address->address->state,
            'postcode' => $address->address->postal,
            'country' => (strlen($address->address->country) > 2)
                ? Lang::getAlpha2Code($address->address->country)
                : strtolower($address->address->country),
            'name' => $address->name
        ];

		if ($additionalFields) {
			$data['point_id'] = $address->pointId ?? '';
		}

		return $data;
    }

    public static function destroyCart() :void
    {
        $cart = \Context::getContext()->cart;
        $cartProducts = $cart->getProducts();

        foreach ($cartProducts as $product) {
            $cart->deleteProduct($product["id_product"], $product['id_product_attribute']);
        }
    }

    public static function addDiscount(CheckoutOrder $calcOrder, $products, $user, $shippingMethodId, string $merchantId)
    {
        $cart = self::createNewCart($products, $user);
        $idCarrier = self::getCarrierId($shippingMethodId);

        $totalDiscountsTaxExcl = (float)abs($cart->getOrderTotal(false, \Cart::ONLY_DISCOUNTS, $cart->getProducts(), $idCarrier));
        $totalDiscountsTaxIncl = (float)abs($cart->getOrderTotal(true, \Cart::ONLY_DISCOUNTS, $cart->getProducts(), $idCarrier));

        if ($totalDiscountsTaxIncl > 0) {
            $price = new Price();
            $price->amount = $totalDiscountsTaxIncl;
            $price->currency = \Context::getContext()->currency->iso_code;
            $price->setTaxEnabled((bool) \Configuration::get('PS_TAX'));
            $price->setHasTax((bool) \Configuration::get('PS_TAX'));
            $calcOrder->discount = $price;
        }

        return $calcOrder;
    }

    private static function getCarrier(string $shippingMethodId)
    {
        $context = \Context::getContext();

        // Carrier start
        $carrier = null;
        $shippingMethods = $context->controller->get('cashbene.payment.configuration')->getShippingMethods();

        if (isset($shippingMethods[$shippingMethodId])) {
            $carrier = new \Carrier($shippingMethods[$shippingMethodId], $context->cookie->id_lang);
            if (boolval($carrier->deleted) === true) {
                return null;
            }
        }

        return $carrier;
    }

    private static function getCarrierId(string $shippingMethodId)
    {
        // Carrier start
        $idCarrier = null;

        if ($carrier = self::getCarrier($shippingMethodId)) {
            $idCarrier = $carrier->id;
        }

        return $idCarrier;
    }
}
