<?php

namespace Cashbene\GatewayPrestashop\Service;

use Context;
use Cashbene\Core\Utils\Configuration as CashbeneConfiguration;
use PrestaShop\PrestaShop\Adapter\Configuration as PrestashopConfiguration;

class ConfigurationService
{
    /** @var array $options */
    private $options;
    /** @var array $shippingMethods */
    private $shippingMethods;

    const SHIPPING_METHOD_KEY = "CASHBENE_PAYMENT_GATEWAY_SHIPPING_METHODS_";

    public function __construct(PrestashopConfiguration $configuration)
    {
        $this->reloadConfigurationSettings();
    }

    public function reloadConfigurationSettings()
    {
        $settings = ModuleSettingsService::getModuleSettings();
        $this->shippingMethods = $this->prepareShippingMethods($settings['shipping_methods'] ?? []);

        $this->options = [
            'environment'   => $settings['CASHBENE_PAYMENT_GATEWAY_ENVIRONMENT'],
            'merchant_id'   => $settings['CASHBENE_PAYMENT_GATEWAY_MERCHANT_ID'],
            'client_id'     => $settings['CASHBENE_PAYMENT_GATEWAY_CLIENT_ID'],
            'client_secret' => $settings['CASHBENE_PAYMENT_GATEWAY_CLIENT_SECRET'],
            'secret_key'    => $settings['CASHBENE_PAYMENT_GATEWAY_SECRET_KEY'],
            'language'      => str_replace('-', '_', Context::getContext()->language->locale)
        ];
    }

    private function prepareShippingMethods(array $shippingMethods): array
    {
        $output = [];
        foreach ($shippingMethods as $key => $shippingMethod) {
            $output[str_replace(self::SHIPPING_METHOD_KEY, '', $key)] = $shippingMethod;
        }

        return $output;
    }

    public function getConfiguration(): CashbeneConfiguration
    {
        $configuration = [
            'merchant_id' => null,
            'client_id' => null,
            'client_secret' => null,
            'secret_key' => null,
            'environment' => null,
            'language' => 'pl_PL'
        ];

        $configuration = array_merge($configuration, $this->options ?: []);

        return new CashbeneConfiguration($configuration);
    }

    public function getShippingMethods(): array
    {
        return $this->shippingMethods;
    }
}
