<?php

namespace Cashbene\GatewayPrestashop\Service;

use Cashbene\Core\Dto\OAuth\AccessToken;
use Cashbene\Core\Gateway;
use Configuration;

class GatewayService
{
    /** @var AccessToken $accessToken */
    private $accessToken;

    /** @var Gateway */
    private static $instance;

    public function __construct(ConfigurationService $configurationService)
    {
        if (self::$instance == null) {
            self::$instance = new Gateway($configurationService->getConfiguration(), $this->getAccessToken());
            self::$instance->setMerchantCredentialsCallback([$this, 'setAccessToken']);
        }
    }

    /**
     * @return Gateway
     */
    public function getGateway(): Gateway
    {
        return self::$instance;
    }

    public function reloadGatewayInstance($configurationService)
    {
        self::$instance = new Gateway($configurationService->getConfiguration(), $this->getAccessToken());;
        self::$instance->setMerchantCredentialsCallback([$this, 'setAccessToken']);
    }

    /**
     * @return AccessToken|null
     */
    public function getAccessToken(): ?AccessToken
    {
        if ($this->accessToken == null) {
            $this->accessToken = Configuration::get('CASHBENE_PAYMENT_GATEWAY_ACCESS_TOKEN');
            $this->accessToken = $this->accessToken ? unserialize($this->accessToken) : null;
        }

        return $this->accessToken;
    }

    /**
     * @param AccessToken $accessToken
     * @return void
     */
    public function setAccessToken(AccessToken $accessToken)
    {
        Configuration::updateValue('CASHBENE_PAYMENT_GATEWAY_ACCESS_TOKEN', serialize($accessToken));
        $this->accessToken = $accessToken;
    }
}
