<?php

namespace Cashbene\GatewayPrestashop\Service;

use Cashbene\Core\Dto\Regulation\MerchantRegulation;
use Configuration;
use CMS;
use Context;
use Validate;

class RegulationService {

    /**
     * Stream pdf file to browser.
     * Can't be escaped because it's a pdf file. It's loaded from trusted external api.
     *
     * @param $pdfContent
     * @param $name
     */
    static public function decodePdfBase64ToFile($pdfContent, $name) {
        $pdfToStream = base64_decode($pdfContent);

        header("Content-type: application/octet-stream");
        header("Content-Type: application/pdf");
        header("Content-Disposition: inline; filename=$name.pdf");
        echo $pdfToStream;
    }

    /**
     * @param string $key
     * @return null|array
     */
    static public function getMerchantTerm(string $key)
    {
        $regulationKeys = [
            MerchantRegulation::TYPE_TERMS_AND_CONDITIONS_PAGE => 'CASHBENE_PAYMENT_GATEWAY_TERMS_AND_CONDITIONS_PAGE',
            MerchantRegulation::TYPE_PRIVACY_POLICY_PAGE => 'CASHBENE_PAYMENT_GATEWAY_PRIVACY_POLICY_PAGE'
        ];

        $settings = json_decode(Configuration::get(ModuleSettingsService::CASHBENE_SETTINGS), true);
        // Language id
        $id_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        // Load CMS Page object
        $post = new CMS($settings[$regulationKeys[$key]], $id_lang);

        // Validate CMS Page object
        if (Validate::isLoadedObject($post)) {
            return [
                'id' => $post->id,
                'title' => $post->meta_title,
                'url' => Context::getContext()->link->getCMSLink($post)
            ];
        }

        return null;
    }
}
